package acme

import (
	"acme-mana/src/common"
	"acme-mana/src/conf"
	"crypto"
	"crypto/ecdsa"
	"crypto/elliptic"
	"crypto/rand"
	"crypto/x509"
	"encoding/json"
	"encoding/pem"
	"github.com/go-acme/lego/v4/certificate"
	"github.com/go-acme/lego/v4/challenge"
	"github.com/go-acme/lego/v4/lego"
	"github.com/go-acme/lego/v4/log"
	"github.com/go-acme/lego/v4/providers/dns/alidns"
	"github.com/go-acme/lego/v4/registration"
	"os"
	"path"
	"path/filepath"
)

func Apply(cert *conf.CertConf) {
	email := cert.Email
	host := cert.Host
	name := cert.Name

	privateKey, err := ecdsa.GenerateKey(elliptic.P256(), rand.Reader)
	if err != nil {
		log.Fatal(err)
	}

	acmeUser := &AcmeUser{
		Email: email,
		key:   privateKey,
	}
	config := lego.NewConfig(acmeUser)
	client, err := lego.NewClient(config)
	if err != nil {
		log.Fatal(err)
	}

	providerName := cert.Provider
	p, _ := conf.FindProvider(providerName)

	provider := getProvider(p)

	//var provider challenge.Provider
	//
	//switch p.Type {
	//case "ali":
	//	conf := alidns.NewDefaultConfig()
	//	conf.RegionID = p.Conf["RegionID"]
	//	conf.APIKey = p.Conf["APIKey"]
	//	conf.SecretKey = p.Conf["SecretKey"]
	//	provider, err = alidns.NewDNSProviderConfig(conf)
	//	if err != nil {
	//		log.Fatal(err)
	//	}
	//}

	chall := client.Challenge
	err = chall.SetDNS01Provider(provider)
	if err != nil {
		return
	}

	registrar := client.Registration
	reg, err := registrar.Register(registration.RegisterOptions{TermsOfServiceAgreed: true})
	if err != nil {
		log.Fatal(err)
	}
	acmeUser.Registration = reg

	request := certificate.ObtainRequest{
		Domains: host,
		Bundle:  true,
	}

	res, err := client.Certificate.Obtain(request)
	if err != nil {
		log.Fatal(err)
	}

	saveCertFile(res, cert, name)

}

//func Apply1(domain Domain) {
//	email, hosts, name := domain.Email, domain.Host, domain.Name
//	privateKey, err := ecdsa.GenerateKey(elliptic.P256(), rand.Reader)
//	if err != nil {
//		log.Fatal(err)
//	}
//
//	acmeUser := &AcmeUser{
//		Email: email,
//		key:   privateKey,
//	}
//
//	config := lego.NewConfig(acmeUser)
//
//	client, err := lego.NewClient(config)
//	if err != nil {
//		log.Fatal(err)
//	}
//
//	ali := appConfig.Provider.Ali
//	conf := alidns.NewDefaultConfig()
//	conf.RegionID = ali.RegionID
//	conf.APIKey = ali.APIKey
//	conf.SecretKey = ali.SecretKey
//	provider, err := alidns.NewDNSProviderConfig(conf)
//	if err != nil {
//		log.Fatal(err)
//	}
//
//	challenge := client.Challenge
//	err = challenge.SetDNS01Provider(provider)
//	if err != nil {
//		return
//	}
//
//	registrar := client.Registration
//	reg, err := registrar.Register(registration.RegisterOptions{TermsOfServiceAgreed: true})
//	if err != nil {
//		log.Fatal(err)
//	}
//	acmeUser.Registration = reg
//
//	request := certificate.ObtainRequest{
//		Domains: hosts,
//		Bundle:  true,
//	}
//
//	cert, err := client.Certificate.Obtain(request)
//	if err != nil {
//		log.Fatal(err)
//	}
//
//	saveCertFile(cert, name)
//}

func getProvider(p *conf.ProviderConf) challenge.Provider {

	switch p.Type {
	case "ali":
		return aliProvider(p)
	}
	return nil
}
func aliProvider(p *conf.ProviderConf) challenge.Provider {
	aliConf := alidns.NewDefaultConfig()
	aliConf.RegionID = p.Conf["RegionID"]
	aliConf.APIKey = p.Conf["APIKey"]
	aliConf.SecretKey = p.Conf["SecretKey"]
	provider, err := alidns.NewDNSProviderConfig(aliConf)
	if err != nil {
		log.Fatal(err)
	}
	return provider
}
func tencentProvider() challenge.Provider {
	return nil
}

// saveCertFile 保存证书文件
func saveCertFile(cert *certificate.Resource, cerfConf *conf.CertConf, name string) {
	dir := cerfConf.Dir
	dir = filepath.Join(dir, name)
	_, err := os.Stat(dir)
	if os.IsNotExist(err) {
		err := os.MkdirAll(dir, 0755)
		if err != nil {
			log.Infof("创建目录 %s 失败", dir)
			log.Fatal(err)
		}
		log.Infof("创建目录 %s", dir)
	}

	certBytes := cert.Certificate
	err = os.WriteFile(path.Join(dir, common.CertFileName), certBytes, 0755)
	if err != nil {
		log.Fatalf("Failed to save certificate: %v", err)
	}

	err = os.WriteFile(path.Join(dir, common.KeyFileName), cert.PrivateKey, 0755)
	if err != nil {
		log.Fatalf("Failed to save private key: %v", err)
	}

	block, _ := pem.Decode(certBytes)
	if block == nil {
		log.Fatalf("Failed to decode PEM block")
		return
	}

	certParse, err := x509.ParseCertificate(block.Bytes)
	if err != nil {
		log.Fatalf("Failed to parse certificate: %v", err)
	}
	certInfo := CertInfo{
		Cert: *cert,
		Info: *certParse,
	}
	// 将 cert 转换为json格式并保存到文件中
	certJson, err := json.Marshal(certInfo)
	if err != nil {
		log.Fatalf("Failed to marshal certificate: %v", err)
	}
	err = os.WriteFile(path.Join(dir, common.CertInfoFileName), certJson, 0644)
	if err != nil {
		log.Fatalf("Failed to save certificate info: %v", err)
	}

}

type AcmeUser struct {
	Email        string
	Registration *registration.Resource
	key          crypto.PrivateKey
}

func (u AcmeUser) GetEmail() string {
	return u.Email
}
func (u AcmeUser) GetRegistration() *registration.Resource {
	return u.Registration
}
func (u AcmeUser) GetPrivateKey() crypto.PrivateKey {
	return u.key
}

type CertInfo struct {
	Cert certificate.Resource
	Info x509.Certificate
}
